# coding=utf-8
"""
装饰器对象，用来装饰同步函数，将同步函数变为可运行的异步函数。


import asyncio
from asfunc import asfunc


@asfunc()
def some_func():
    print("func run.")


asyncio.run(some_func())
"""

__version__ = "0.0.1"

__all__ = ["asfunc"]


import asyncio
import functools


class AsFunc:
    """装饰器对象，用来装饰同步函数，将同步函数变为可运行的异步函数。"""
    def __init__(self):
        self._func_coroutine = None  # 存放装饰的异步函数协程对象

    async def run_func(self):
        """用于__await__执行"""
        return await self._func_coroutine

    def __await__(self):
        """
        必须定义这个方法才能直接 await 这个类的对象
        并且，返回值必须是一个 iterator，这里直接
        使用 async 函数的内置方法 __await__()
        """
        return self.run_func().__await__()

    def __call__(self, func):
        """同步函数装饰器: 装饰同步函数
        """

        @functools.wraps(func)
        async def func_magic_async(*args, **kwargs):
            if asyncio.iscoroutinefunction(func):
                self._func_coroutine = func(*args, **kwargs)
            else:
                self._func_coroutine = asyncio.to_thread(func, *args, **kwargs)
            await self
        return func_magic_async


asfunc = AsFunc

